@@ -16,7 +16,7 @@ require HTML::DOM::Node;
 require HTML::DOM::NodeList::Magic;
 
 our @ISA = qw'HTML::DOM::Node';
-our $VERSION = '0.011';
+our $VERSION = '0.012';
 
 
 {
@@ -187,7 +187,7 @@ sub id { # unfivetuninely, SUPER::id($something) doesn't return the old val
 
 sub title { shift->attr(title => @_) }
 sub lang  { shift->attr(lang  => @_) }
-sub dir   { shift->attr(dir   => @_) }
+sub dir   { lc shift->attr(dir   => @_) }
 sub className { shift->attr(class => @_) }
 
 =item style
@@ -232,35 +232,84 @@ sub content_offset {
 
 =head2 Other Methods
 
-See the DOM spec. for descriptions.
-
 =over 4
 
-=item getAttribute
+=item getAttribute ( $name )
 
-=item setAttribute
+Returns the attribute's value as a string.
 
-=item removeAttribute
+=item setAttribute ( $name, $value )
 
-=item getAttributeNode
+Sets the attribute named C<$name> to C<$value>.
 
-=item setAttributeNode
+=item removeAttribute ( $name )
 
-=item removeAttributeNode
+Deletes the C<$name>d attribute.
 
-=item getElementsByTagName
+=item getAttributeNode ( $name )
 
-=item hasAttribute
+Returns an attribute node (L<HTML::DOM::Attr>).
+
+=item setAttributeNode ( $attr )
+
+Sets the attribute whose name is C<< $attr->nodeName >> to the attribute
+object itself. If it replaces another attribute object, the latter is
+returned.
+
+=item removeAttributeNode ( $attr )
+
+Removes and returns the C<$attr>.
+
+=item getElementsByTagName ( $tagname)
+
+This finds all elements with that tag name under the current element,
+returning them as a list in list context or a node list object in scalar
+context.
+
+=item hasAttribute ( $name )
+
+Returns true or false, indicating whether this element has an attribute
+named C<$name>, even one that is implied.
 
 =back
 
 =cut
 
+my %attr_defaults = (
+	br => { clear => 'none' },
+	td => { colspan => '1', rowspan=>1},
+	th => { colspan =>  1,  rowspan=>1},
+	form => {
+		enctype => 'application/x-www-form-urlencoded',
+		method => 'GET',
+	},
+	frame =>{frameborder  => 1,scrolling=> 'auto'},
+	iframe=> {frameborder => 1,scrolling=>'auto'},
+	'area'=> {'shape'         => 'rect',},
+	'a' =>{'shape'            => 'rect',},
+	'col'=>{ 'span'           =>  1,},
+	'colgroup'=>{ 'span'      =>  1,},
+	'input',{ 'type'         => 'TEXT',},
+	'button' =>{'type'        => 'submit',},
+	'param' =>{'valuetype'    => 'DATA'},
+);
+# Note: The _HTML_DOM_unspecified key used below points to a hash that
+#       stores Attr objects for implicit attributes in this list.
+
 sub getAttribute {
 	if ($_[1] eq 'style' and $_[0]{_HTML_DOM_style}) {
 		$_[0]{_HTML_DOM_style}->cssText
 	} else {
-		''.($_[0]->attr($_[1])||'')
+		''.($_[0]->attr($_[1])||do{
+			my $tag = $_[0]->tag;
+			return '' unless exists $attr_defaults{$tag}
+				and exists $attr_defaults{$tag}{$_[1]}
+				or $tag eq 'html' and $_[1] eq 'version'
+				   and exists $_[0]->{_HTML_DOM_version};
+			$_[1] eq 'version'
+				? $_[0]->{_HTML_DOM_version}
+				: $attr_defaults{$tag}{$_[1]}
+		})
 	}
 }
 
@@ -305,9 +354,27 @@ sub removeAttribute {
 sub getAttributeNode {
 	my $elem = shift;
 	my $name = lc shift;
-	defined(my $attr = $elem->attr($name))
+
+	my $attr = $elem->attr($name);
+	unless(defined $attr
 		or $name eq 'style' and $elem->{_HTML_DOM_style}
-		or return;
+	) { # check to see whether it has a default value
+		my $tag = $elem->tag;
+		return $elem->{_HTML_DOM_unspecified}{$name} ||= do{
+			return unless exists $attr_defaults{$tag}
+				and exists $attr_defaults{$tag}{$name}
+				or $tag eq 'html' and $name eq 'version'
+				   and exists $elem->{_HTML_DOM_version};
+			my $attr = HTML::DOM::Attr->new($name);
+			$attr->_set_ownerDocument($elem->ownerDocument);
+			$attr->_element($elem);
+			$attr->value($name eq 'version'
+				? $elem->{_HTML_DOM_version}
+				: $attr_defaults{$tag}{$name});
+			$attr;
+		};
+	}
+
 	if(!ref $attr) {
 		$elem->attr($name, my $new_attr =
 			HTML::DOM::Attr->new($name));
@@ -380,12 +447,16 @@ sub setAttributeNode {
 sub removeAttributeNode {
 	my($elem,$attr) = @_;
 
-	refaddr $attr == refaddr $elem->attr(my $name = $attr->nodeName)
-		or die HTML::DOM::Exception->new(NOT_FOUND_ERR,
+	my $old_val = $elem->attr(my $name = $attr->nodeName);
+	defined($old_val)
+		? ref$old_val && refaddr $attr == refaddr $old_val
+		: exists $elem->{_HTML_DOM_unspecified}{$name}
+	or die HTML::DOM::Exception->new(NOT_FOUND_ERR,
 		"The node passed to removeAttributeNode is not an " .
 		"attribute of this element.");
 
 	$elem->attr($name, undef);
+	delete $elem->{_HTML_DOM_unspecified}{$name};
 	$attr->_element(undef);
 	lc $name eq 'style' and delete $elem->{_HTML_DOM_style};
 	return $attr
@@ -410,7 +481,17 @@ sub getElementsByTagName { # very similar to the one in HTML::DOM
 	}
 }
 
-sub hasAttribute { defined shift->attr(shift) }
+sub hasAttribute {
+	my ($self,$attrname)= (shift, lc shift);
+	my $tag;
+	defined $self->attr($attrname)
+		or exists $attr_defaults{$tag = $self->tag}
+			and exists $attr_defaults{$tag}{$attrname}
+		or $tag eq 'html' and $attrname eq 'version'
+			and exists $self->{_HTML_DOM_version}
+}
+
+sub _attr_specified { defined shift->attr(shift) }
 
 # ------- OVERRIDDEN NODE METHDOS ---------- #
 
@@ -461,21 +542,21 @@ L<HTML::DOM/CLASSES AND DOM INTERFACES>
 # ------- HTMLHtmlElement interface ---------- #
 
 package HTML::DOM::Element::HTML;
-our $VERSION = '0.011';
+our $VERSION = '0.012';
 our @ISA = 'HTML::DOM::Element';
 sub version { shift->attr('version' => @_) }
 
 # ------- HTMLHeadElement interface ---------- #
 
 package HTML::DOM::Element::Head;
-our $VERSION = '0.011';
+our $VERSION = '0.012';
 our @ISA = 'HTML::DOM::Element';
 sub profile { shift->attr('profile' => @_) }
 
 # ------- HTMLLinkElement interface ---------- #
 
 package HTML::DOM::Element::Link;
-our $VERSION = '0.011';
+our $VERSION = '0.012';
 our @ISA = 'HTML::DOM::Element';
 sub disabled {
 	if(@_ > 1) {
@@ -534,7 +615,7 @@ sub removeAttributeNode {
 # ------- HTMLTitleElement interface ---------- #
 
 package HTML::DOM::Element::Title;
-our $VERSION = '0.011';
+our $VERSION = '0.012';
 our @ISA = 'HTML::DOM::Element';
 # This is what I call FWP (no lexical vars):
 sub text {
@@ -549,7 +630,7 @@ sub text {
 # ------- HTMLMetaElement interface ---------- #
 
 package HTML::DOM::Element::Meta;
-our $VERSION = '0.011';
+our $VERSION = '0.012';
 our @ISA = 'HTML::DOM::Element';
 sub content   { shift->attr('content'    => @_) }
 sub httpEquiv { shift->attr('http-equiv' => @_) }
@@ -559,7 +640,7 @@ sub scheme    { shift->attr('scheme'     => @_) }
 # ------- HTMLBaseElement interface ---------- #
 
 package HTML::DOM::Element::Base;
-our $VERSION = '0.011';
+our $VERSION = '0.012';
 our @ISA = 'HTML::DOM::Element';
 *href =\& HTML::DOM::Element::Link::href;
 *target =\& HTML::DOM::Element::Link::target;
@@ -567,7 +648,7 @@ our @ISA = 'HTML::DOM::Element';
 # ------- HTMLIsIndexElement interface ---------- #
 
 package HTML::DOM::Element::IsIndex;
-our $VERSION = '0.011';
+our $VERSION = '0.012';
 our @ISA = 'HTML::DOM::Element';
 sub form     { (shift->look_up(_tag => 'form'))[0] || () }
 sub prompt   { shift->attr('prompt'  => @_) }
@@ -575,7 +656,7 @@ sub prompt   { shift->attr('prompt'  => @_) }
 # ------- HTMLStyleElement interface ---------- #
 
 package HTML::DOM::Element::Style;
-our $VERSION = '0.011';
+our $VERSION = '0.012';
 our @ISA = 'HTML::DOM::Element';
 *disabled = \&HTML::DOM::Element::Link::disabled;
 *media =\& HTML::DOM::Element::Link::media;
@@ -595,7 +676,7 @@ sub sheet {
 # ------- HTMLBodyElement interface ---------- #
 
 package HTML::DOM::Element::Body;
-our $VERSION = '0.011';
+our $VERSION = '0.012';
 our @ISA = 'HTML::DOM::Element';
 sub aLink      { shift->attr( aLink      => @_) }
 sub background { shift->attr( background => @_) }
@@ -613,15 +694,15 @@ sub vLink      { shift->attr('vlink'     => @_) }
 # ------- HTMLUListElement interface ---------- #
 
 package HTML::DOM::Element::UL;
-our $VERSION = '0.011';
+our $VERSION = '0.012';
 our @ISA = 'HTML::DOM::Element';
 sub compact { shift->attr( compact => @_) }
-*type =\& HTML::DOM::Element::Link::type;
+sub type { lc shift->attr( type => @_) }
 
 # ------- HTMLOListElement interface ---------- #
 
 package HTML::DOM::Element::OL;
-our $VERSION = '0.011';
+our $VERSION = '0.012';
 our @ISA = 'HTML::DOM::Element';
 sub start { shift->attr( start => @_) }
 *compact=\&HTML::DOM::Element::UL::compact;
@@ -630,28 +711,28 @@ sub start { shift->attr( start => @_) }
 # ------- HTMLDListElement interface ---------- #
 
 package HTML::DOM::Element::DL;
-our $VERSION = '0.011';
+our $VERSION = '0.012';
 our @ISA = 'HTML::DOM::Element';
 *compact=\&HTML::DOM::Element::UL::compact;
 
 # ------- HTMLDirectoryElement interface ---------- #
 
 package HTML::DOM::Element::Dir;
-our $VERSION = '0.011';
+our $VERSION = '0.012';
 our @ISA = 'HTML::DOM::Element';
 *compact=\&HTML::DOM::Element::UL::compact;
 
 # ------- HTMLMenuElement interface ---------- #
 
 package HTML::DOM::Element::Menu;
-our $VERSION = '0.011';
+our $VERSION = '0.012';
 our @ISA = 'HTML::DOM::Element';
 *compact=\&HTML::DOM::Element::UL::compact;
 
 # ------- HTMLLIElement interface ---------- #
 
 package HTML::DOM::Element::LI;
-our $VERSION = '0.011';
+our $VERSION = '0.012';
 our @ISA = 'HTML::DOM::Element';
 *type =\& HTML::DOM::Element::Link::type;
 sub value { shift->attr( value => @_) }
@@ -659,49 +740,49 @@ sub value { shift->attr( value => @_) }
 # ------- HTMLDivElement interface ---------- #
 
 package HTML::DOM::Element::Div;
-our $VERSION = '0.011';
+our $VERSION = '0.012';
 our @ISA = 'HTML::DOM::Element';
-sub align { shift->attr( align => @_) }
+sub align { lc shift->attr( align => @_) }
 
 # ------- HTMLParagraphElement interface ---------- #
 
 package HTML::DOM::Element::P;
-our $VERSION = '0.011';
+our $VERSION = '0.012';
 our @ISA = 'HTML::DOM::Element';
 *align =\& HTML::DOM::Element::Div::align;
 
 # ------- HTMLHeadingElement interface ---------- #
 
 package HTML::DOM::Element::Heading;
-our $VERSION = '0.011';
+our $VERSION = '0.012';
 our @ISA = 'HTML::DOM::Element';
 *align =\& HTML::DOM::Element::Div::align;
 
 # ------- HTMLQuoteElement interface ---------- #
 
 package HTML::DOM::Element::Quote;
-our $VERSION = '0.011';
+our $VERSION = '0.012';
 our @ISA = 'HTML::DOM::Element';
 sub cite { shift->attr( cite => @_) }
 
 # ------- HTMLPreElement interface ---------- #
 
 package HTML::DOM::Element::Pre;
-our $VERSION = '0.011';
+our $VERSION = '0.012';
 our @ISA = 'HTML::DOM::Element';
 sub width { shift->attr( width => @_) }
 
 # ------- HTMLBRElement interface ---------- #
 
 package HTML::DOM::Element::Br;
-our $VERSION = '0.011';
+our $VERSION = '0.012';
 our @ISA = 'HTML::DOM::Element';
-sub clear { shift->attr( clear => @_) }
+sub clear { lc shift->attr( clear => @_) }
 
 # ------- HTMLBaseFontElement interface ---------- #
 
 package HTML::DOM::Element::BaseFont;
-our $VERSION = '0.011';
+our $VERSION = '0.012';
 our @ISA = 'HTML::DOM::Element';
 sub color { shift->attr( color => @_) }
 sub face  { shift->attr( face  => @_) }
@@ -710,7 +791,7 @@ sub size  { shift->attr( size  => @_) }
 # ------- HTMLBaseFontElement interface ---------- #
 
 package HTML::DOM::Element::Font;
-our $VERSION = '0.011';
+our $VERSION = '0.012';
 our @ISA = 'HTML::DOM::Element';
 *color =\& HTML::DOM::Element::BaseFont::color;
 *face =\& HTML::DOM::Element::BaseFont::face;
@@ -719,7 +800,7 @@ our @ISA = 'HTML::DOM::Element';
 # ------- HTMLHRElement interface ---------- #
 
 package HTML::DOM::Element::HR;
-our $VERSION = '0.011';
+our $VERSION = '0.012';
 our @ISA = 'HTML::DOM::Element';
 *align =\& HTML::DOM::Element::Div::align;
 sub noShade  { shift->attr( noshade  => @_) }
@@ -729,7 +810,7 @@ sub noShade  { shift->attr( noshade  => @_) }
 # ------- HTMLModElement interface ---------- #
 
 package HTML::DOM::Element::Mod;
-our $VERSION = '0.011';
+our $VERSION = '0.012';
 our @ISA = 'HTML::DOM::Element';
 *cite =\& HTML::DOM::Element::Quote::cite;
 sub dateTime  { shift->attr( datetime  => @_) }
@@ -737,7 +818,7 @@ sub dateTime  { shift->attr( datetime  => @_) }
 # ------- HTMLAnchorElement interface ---------- #
 
 package HTML::DOM::Element::A;
-our $VERSION = '0.011';
+our $VERSION = '0.012';
 our @ISA = 'HTML::DOM::Element';
 sub accessKey  { shift->attr(               accesskey  => @_) }
 *   charset    =\&HTML::DOM::Element::Link::charset           ;
@@ -758,7 +839,7 @@ sub focus { shift->trigger_event('focus') }
 # ------- HTMLImageElement interface ---------- #
 
 package HTML::DOM::Element::Img;
-our $VERSION = '0.011';
+our $VERSION = '0.012';
 our @ISA = 'HTML::DOM::Element';
 sub lowSrc  { shift->attr(               lowsrc  => @_) }
 *   name  = \&HTML::DOM::Element::Meta::name            ;
@@ -777,7 +858,7 @@ sub vspace   { shift->attr(              vspace   => @_) }
 # ------- HTMLObjectElement interface ---------- #
 
 package HTML::DOM::Element::Object;
-our $VERSION = '0.011';
+our $VERSION = '0.012';
 our @ISA = 'HTML::DOM::Element';
 *form=\&HTML::DOM::Element::IsIndex::form;
 sub code  { shift->attr(               code  => @_) }
@@ -801,17 +882,17 @@ sub tabIndex      { shift->attr(              tabindex      => @_) }
 # ------- HTMLParamElement interface ---------- #
 
 package HTML::DOM::Element::Param;
-our $VERSION = '0.011';
+our $VERSION = '0.012';
 our @ISA = 'HTML::DOM::Element';
 *name=\&HTML::DOM::Element::Meta::name;
 *type=\&HTML::DOM::Element::Link::type;
 *value=\&HTML::DOM::Element::LI::value;
-sub valueType{shift->attr(valuetype=>@_)}
+sub valueType{lc shift->attr(valuetype=>@_)}
 
 # ------- HTMLAppletElement interface ---------- #
 
 package HTML::DOM::Element::Applet;
-our $VERSION = '0.011';
+our $VERSION = '0.012';
 our @ISA = 'HTML::DOM::Element';
 * align    = \ & HTML::DOM::Element::Div::align       ;
 * alt      = \ & HTML::DOM::Element::Img::alt         ;
@@ -828,7 +909,7 @@ sub object { shift -> attr ( object => @_ ) }
 # ------- HTMLMapElement interface ---------- #
 
 package HTML::DOM::Element::Map;
-our $VERSION = '0.011';
+our $VERSION = '0.012';
 our @ISA = 'HTML::DOM::Element';
 sub areas { # ~~~ I need to make this cache the resulting collection obj
 	my $self = shift;
@@ -849,7 +930,7 @@ sub areas { # ~~~ I need to make this cache the resulting collection obj
 # ------- HTMLAreaElement interface ---------- #
 
 package HTML::DOM::Element::Area;
-our $VERSION = '0.011';
+our $VERSION = '0.012';
 our @ISA = 'HTML::DOM::Element';
 * accessKey = \ & HTML::DOM::Element::A::accessKey     ;
 * alt       = \ & HTML::DOM::Element::Img::alt         ;
@@ -863,7 +944,7 @@ sub noHref { shift -> attr ( nohref => @_ ) }
 # ------- HTMLScriptElement interface ---------- #
 
 package HTML::DOM::Element::Script;
-our $VERSION = '0.011';
+our $VERSION = '0.012';
 our @ISA = 'HTML::DOM::Element';
 * text    = \ &HTML::DOM::Element::Title::text   ;
 sub htmlFor { shift -> attr ( for   => @_ )      }
@@ -876,7 +957,7 @@ sub defer   { shift -> attr ( defer => @_ )      }
 # ------- HTMLFrameSetElement interface ---------- #
 
 package HTML::DOM::Element::FrameSet;
-our $VERSION = '0.011';
+our $VERSION = '0.012';
 our @ISA = 'HTML::DOM::Element';
 sub rows { shift -> attr ( rows   => @_ )      }
 sub cols   { shift -> attr ( cols => @_ )      }
@@ -884,30 +965,30 @@ sub cols   { shift -> attr ( cols => @_ )      }
 # ------- HTMLFrameElement interface ---------- #
 
 package HTML::DOM::Element::Frame;
-our $VERSION = '0.011';
+our $VERSION = '0.012';
 our @ISA = 'HTML::DOM::Element';
-sub frameBorder { shift -> attr ( frameBorder  => @_ )      }
+sub frameBorder { lc shift -> attr ( frameBorder  => @_ )      }
 sub longDesc    { shift -> attr ( longdesc     => @_ )      }
 sub marginHeight{ shift -> attr ( marginheight => @_ )      }
 sub marginWidth { shift -> attr ( marginwidth  => @_ )      }
 * name    = \ &HTML::DOM::Element::Meta::name   ;
 sub noResize    { shift -> attr ( noresize     => @_ )      }
-sub scrolling   { shift -> attr ( scrolling    => @_ )      }
+sub scrolling   { lc shift -> attr ( scrolling    => @_ )      }
 * src     = \ &HTML::DOM::Element::Img::src     ;
 
 # ------- HTMLIFrameElement interface ---------- #
 
 package HTML::DOM::Element::IFrame;
-our $VERSION = '0.011';
+our $VERSION = '0.012';
 our @ISA = 'HTML::DOM::Element';
 *align  = \&HTML::DOM::Element::Div::align;
-sub frameBorder { shift -> attr ( frameBorder  => @_ )      }
+*frameBorder = \&HTML::DOM::Element::Frame::frameBorder;
 *height = \&HTML::DOM::Element::Img::height;
 sub longDesc    { shift -> attr ( longdesc     => @_ )      }
 sub marginHeight{ shift -> attr ( marginheight => @_ )      }
 sub marginWidth { shift -> attr ( marginwidth  => @_ )      }
 *name   = \&HTML::DOM::Element::Meta::name;
-sub scrolling   { shift -> attr ( scrolling    => @_ )      }
+*scrolling = \&HTML::DOM::Element::Frame::scrolling;
 *src    = \&HTML::DOM::Element::Img::src;
 *width  = \&HTML::DOM::Element::Pre::width;
 
